#include <cstdio>
#include <GL/gl.h>
#include <GL/glu.h>

#include "wordz.h"

const char* FONT_CHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789`~!@#$%^&*()-_=+[{]}\\|;:'\",<.>/?";

int char_padding_h;
int char_padding_v;

float char_w = 0.0403f;
float char_h = 0.0666f;
float printed_char_w = 0.0277;

SDL_Surface** LoadFont(char* filename, int char_w, int char_h)
	throw (std::bad_alloc)
{
	SDL_Surface* all = SDL_LoadBMP(filename);
	if (!all) {
		return 0;
	}

	SDL_Surface** font = new SDL_Surface*[NUM_FONT_CHARS];

	Uint32 flags = cfg.has_hw_surfaces ? SDL_HWSURFACE : SDL_SWSURFACE;
	Uint32 rmask;
	Uint32 gmask;
	Uint32 bmask;
	Uint32 amask;

	rmask = all->format->Rmask;
	gmask = all->format->Gmask;
	bmask = all->format->Bmask;
	amask = all->format->Amask;

	for (int i = 0; i < NUM_FONT_CHARS; i++) {
		font[i] = SDL_CreateRGBSurface(flags, 64, 64,
				all->format->BitsPerPixel,
				rmask, gmask, bmask, amask);
		if (!font[i]) {
			// FIXME: clean up
			return 0;
		}
	}

	char_padding_h = (64-char_w)/2;
	char_padding_v = (64-char_h)/2;

	SDL_Rect fillrect = { 0, 0, 64, 64 };
	SDL_Rect dstrect = { char_padding_h, char_padding_v, char_w, char_h };
	SDL_Rect srcrect = { 0, 0, char_w, char_h };

	for (int i = 0; i < NUM_FONT_CHARS; i++) {
		Uint32 color = SDL_MapRGB(font[i]->format, 255, 147, 0);
		SDL_FillRect(font[i], &fillrect, color);
		int ret;
		if ((ret = SDL_BlitSurface(all, &srcrect, font[i], &dstrect))
				< 0)
			printf("blit failed: ret=%d i=%d\n", ret, i);
		srcrect.x += char_w;
	}

	return font;
}

void DeleteFont(SDL_Surface** font)
{
	for (int i = 0; i < NUM_FONT_CHARS; i++) {
	    SDL_FreeSurface(font[i] );
	}
	delete[] font;
}

static int FindCharacterIndex(char c)
{
	for (int i = 0; FONT_CHARS[i]; i++)
		if (FONT_CHARS[i] == c)
			return i;
	return -1;
}

void GLPrintf(int x, int y, char* format, ...)
{
	char text[1000];
	va_list ap;
	va_start(ap, format);
	vsprintf(text, format, ap);
	va_end(ap);

	int tn;
	float xx = ((float)x / (float)SCREEN_WIDTH) * ratio - half_ratio;
	float yy = 0.5f - ((float)y / (float)SCREEN_HEIGHT);

	glLoadIdentity();
	glTranslatef(xx, yy, -2.2f);

	for (int i = 0; text[i]; i++) {
		tn = FindCharacterIndex(text[i]);
		if (tn < 0) {
			glTranslatef(printed_char_w, 0.0f, 0.0f);
			continue;
		}
		glBindTexture(GL_TEXTURE_2D, textures[tn]);
		glBegin(GL_QUADS);
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(0.0f, -char_h, 1.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f(char_w, -char_h, 1.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f(char_w, 0.0f, 1.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(0.0f, 0.0f, 1.0f);
		glEnd();
		glTranslatef(printed_char_w, 0.0f, 0.0f);
	}
}
